/**
 * description
 *
 * @author eson
 *2022年7月12日-11:10:54
 */
package com.yuandian.dataflow.statemachine.rpc;

import java.io.Serializable;

import com.alipay.sofa.jraft.Status;
import com.alipay.sofa.jraft.error.RaftError;
import com.alipay.sofa.jraft.rpc.RpcContext;
import com.alipay.sofa.jraft.rpc.RpcProcessor;
import com.yuandian.dataflow.statemachine.StateFactory;
import com.yuandian.dataflow.statemachine.annotations.WorkerRegister;
import com.yuandian.dataflow.statemachine.closure.GenericClosure;
import com.yuandian.dataflow.statemachine.operate.Operate;

import javassist.ClassPath;
 
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import lombok.extern.slf4j.Slf4j;

/**
 * description
 *
 * @author eson
 *2022年7月12日-11:10:54
 */
@Slf4j
@WorkerRegister
public class OperateProcessor implements RpcProcessor<OperateProcessor.OperateRequest> {

    /**
     * 同步状态时, 需要用的结构类. 新增的状态可以在 Operate结构里添加
     *
     * @author eson
     *2022年7月11日-16:01:07
    */
    @Getter
    @Setter
    @ToString
    public static class OperateRequest  implements Serializable {

        private static final long serialVersionUID = 1L;
    
        private Operate operate; 
    }


    @Override
    public void handleRequest(RpcContext rpcCtx, OperateRequest  request) {
   
        // log.info("request: {}", request);
 
        final GenericClosure  closure = new GenericClosure () {
            @Override
            public void run(Status status) {
                
                if(status.isOk()) {
                    // log.info("{}", status);
                    rpcCtx.sendResponse(getResponse());
                    return;
                }
 
                if(status.getRaftError() == RaftError.EPERM) {
                    //TODO: Not leader 需要转发
                    log.info("{}", status);
                } 
                
            }
        };
    
        StateFactory.getStateServer().applyOperate(request.getOperate(), closure); 
    }

    @Override
    public String interest() {
        return OperateRequest.class.getName();
    }
}
